-- ***************************************************************************
-- *  Copyright (c) 2017 by Michael Fischer (www.emb4fun.de)
-- *  All rights reserved.
-- *
-- *  Redistribution and use in source and binary forms, with or without 
-- *  modification, are permitted provided that the following conditions 
-- *  are met:
-- *  
-- *  1. Redistributions of source code must retain the above copyright 
-- *     notice, this list of conditions and the following disclaimer.
-- *  2. Redistributions in binary form must reproduce the above copyright
-- *     notice, this list of conditions and the following disclaimer in the 
-- *     documentation and/or other materials provided with the distribution.
-- *  3. Neither the name of the author nor the names of its contributors may 
-- *     be used to endorse or promote products derived from this software 
-- *     without specific prior written permission.
-- *
-- *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
-- *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
-- *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
-- *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
-- *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
-- *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
-- *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
-- *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
-- *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
-- *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
-- *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
-- *  SUCH DAMAGE.
-- *
-- ***************************************************************************
-- *  History:
-- *
-- *  30.06.2017  mifi  First version.
-- ***************************************************************************

-- ***************************************************************************
-- *  DEFINE: Library                                                        *
-- ***************************************************************************
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;


-- ***************************************************************************
-- *  DEFINE: Entity                                                         *
-- ***************************************************************************

entity avs2con32_if is
   generic (
      ADDRESS_WIDTH : integer := 3
   );
	port (
      -- Avalon
		clk           : in  std_logic                     := '0';                           --   clock.clk
		reset         : in  std_logic                     := '0';                           --   reset.reset
		avs_address   : in  std_logic_vector(ADDRESS_WIDTH-1 downto 0) := (others => '0');  --      s0.address
		avs_read      : in  std_logic                     := '0';                           --        .read
		avs_readdata  : out std_logic_vector(31 downto 0);                                  --        .readdata
		avs_write     : in  std_logic                     := '0';                           --        .write
		avs_writedata : in  std_logic_vector(31 downto 0) := (others => '0');               --        .writedata
      
      -- Conduit
		address_o     : out std_logic_vector(ADDRESS_WIDTH-1 downto 0);                     -- conduit.export
		read_o        : out std_logic;                                                      --        .export
		readdata_i    : in  std_logic_vector(31 downto 0) := (others => '0');               --        .export
		write_o       : out std_logic;                                                      --        .export
		writedata_o   : out std_logic_vector(31 downto 0)                                   --        .export
	);
end entity avs2con32_if;


-- ***************************************************************************
-- *  DEFINE: Architecture                                                   *
-- ***************************************************************************

architecture syn of avs2con32_if is
begin

	address_o    <= avs_address;
	read_o       <= avs_read;
	avs_readdata <= readdata_i;
	write_o      <= avs_write;
	writedata_o  <= avs_writedata;

end architecture syn;

-- *** EOF ***
